from __future__ import unicode_literals
import re
from datetime import datetime
from cached_property import cached_property
from requests.auth import HTTPBasicAuth
import requests


selected_option = re.compile(r'<option\s*selected="selected"\s*value="(.+?)"\s*>')


def get_selected_value(html, name):
    regex = '<select\s*name="{0}"[^>]*?>\s*(.+?)\s*</select>'.format(name.replace('$', '\$'))
    sub_html = re.findall(regex, html)[0]
    return selected_option.findall(sub_html)[0]


def get_value(html, name):
    regex = '<input\s[^>]*?name="{0}"[^>]*?value="(.+?)"[^>]*?>'.format(name.replace('$', '\$'))
    return re.findall(regex, html)[0]


def merge_two_dicts(x, y):
    """Given two dicts, merge them into a new dict as a shallow copy."""
    z = x.copy()
    z.update(y)
    return z


class TTIntegration(object):

    post_url = 'https://tt.muranosoft.com/TimeEntry.aspx'
    holiday_id = 27

    def __init__(self, username, password, category=2, start_work_day=10, end_work_day=18):
        self.username = username
        self.password = password
        self.category = category
        self.start_work_day = start_work_day
        self.end_work_day = end_work_day

    @cached_property
    def html(self):
        return requests.post(self.post_url, auth=HTTPBasicAuth(self.username, self.password)).text

    def select_day(self, current_date):
        eventargument = (current_date - datetime(2015, 7, 1)).days + 5660
        data = {
            '__EVENTTARGET': 'ctl00$_content$calDatesSelector',
            '__EVENTARGUMENT': eventargument,
            'ctl00$_content$ScriptManager1':
                'ctl00$_content$ctl00$_content$panelEditFormPanel|ctl00$_content$lbtnAddEntry',
            'ctl00$_content$ddlProjects': get_selected_value(self.html, 'ctl00$_content$ddlProjects'),
            'ctl00$_content$ddlCategories': self.category,
            'ctl00$_content$ucStartTime$ddlHour': self.start_work_day,
            'ctl00$_content$ucStartTime$ddlMinutes': 0,
            'ctl00$_content$ucEndTime$ddlHour': self.end_work_day,
            'ctl00$_content$ucEndTime$ddlMinutes': 0,
            'ctl00$_content$ddlUsers': get_selected_value(self.html, 'ctl00$_content$ddlUsers'),
            'RadAJAXControlID': 'ctl00__content_RadAjaxManager1',
            '__VIEWSTATEGENERATOR': get_value(self.html, '__VIEWSTATEGENERATOR'),
            '__VIEWSTATE': get_value(self.html, '__VIEWSTATE'),
            'ctl00$_content$ddlMonths': current_date.month,
            'ctl00$_content$ddlYears': current_date.year,
            'ctl00$_content$ddlDays': current_date.strftime('%d.%m.%Y 0:00:00'),
            'ctl00$_content$tboxDescription': '',
        }
        return requests.post(self.post_url, data, auth=HTTPBasicAuth(self.username, self.password)).text

    def context(self, current_date):
        html = self.select_day(current_date)
        return {
            'ctl00$_content$ScriptManager1':
                'ctl00$_content$ctl00$_content$panelEditFormPanel|ctl00$_content$lbtnAddEntry',
            '__EVENTTARGET': 'ctl00$_content$lbtnAddEntry',
            'ctl00$_content$ddlProjects': get_selected_value(html, 'ctl00$_content$ddlProjects'),
            'ctl00$_content$ddlCategories': self.category,
            'ctl00$_content$ucStartTime$ddlHour': self.start_work_day,
            'ctl00$_content$ucStartTime$ddlMinutes': 0,
            'ctl00$_content$ucEndTime$ddlHour': self.end_work_day,
            'ctl00$_content$ucEndTime$ddlMinutes': 0,
            'ctl00$_content$ddlUsers': get_selected_value(html, 'ctl00$_content$ddlUsers'),
            'RadAJAXControlID': 'ctl00__content_RadAjaxManager1',
            '__VIEWSTATEGENERATOR': get_value(html, '__VIEWSTATEGENERATOR'),
            '__VIEWSTATE': get_value(html, '__VIEWSTATE')
        }

    def post_message(self, current_date, is_working, message):
        data = {
            'ctl00$_content$ddlMonths': current_date.month,
            'ctl00$_content$ddlYears': current_date.year,
            'ctl00$_content$ddlDays': current_date.strftime('%d.%m.%Y 0:00:00'),
            'ctl00$_content$tboxDescription': message,
        }
        if not is_working:
            data['ctl00$_content$tboxDescription'] = 'Holiday'
            data['ctl00$_content$ddlCategories'] = self.holiday_id
        post_data = merge_two_dicts(self.context(current_date), data)
        result = requests.post(self.post_url, post_data, auth=HTTPBasicAuth(self.username, self.password))
        print('{0} - {1} : {2} - {3}'.format(
            result,
            current_date,
            post_data['ctl00$_content$ddlCategories'],
            post_data['ctl00$_content$tboxDescription']))


from __future__ import unicode_literals
from datetime import date, datetime
from functools import partial
import operator
from dateutil.relativedelta import relativedelta
from dateutil.rrule import rrule, DAILY, MO, TU, WE, TH, FR
import requests


calened_url = 'http://basicdata.ru/api/json/calend/'

date_format = "%Y-%m-%d"


def mark_holidays(cal_dates, current_date):
    is_working = (cal_dates
                  .get(current_date.year, {})
                  .get(current_date.month, {})
                  .get(current_date.day, {})
                  .get('isWorking', 0))
    return current_date, not is_working


def get_worked_days(start_date=None, end_date=None):
    if start_date is None:
        start_date = date.today()+relativedelta(day=1)
    else:
        start_date = datetime.strptime(start_date, date_format).date()
    if end_date is None:
        end_date = date.today()+relativedelta(months=+1, day=1) + relativedelta(days=-1)
    else:
        end_date = datetime.strptime(end_date, date_format).date()
    response = requests.get(calened_url)
    cal_dates = response.json()
    dates = rrule(DAILY,
                  dtstart=start_date,
                  until=end_date,
                  byweekday=(MO, TU, WE, TH, FR))
    usual_worked_days = list(map(partial(mark_holidays, cal_dates), dates))
    today = date.today()
    days = (cal_dates.get(today.year, {}).get(today.month, {}))
    worked_dates = [(datetime(today.year, today.month, day), True) for day in days if days.get(day).get('isWorking') == 0]
    return list(sorted(usual_worked_days + worked_dates, key=operator.itemgetter(0)))


#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


requirements = [
    'GitPython',
    'six',
    'requests',
    'python-dateutil',
    'cached-property'
]

setup(
    name="tta",
    version="0.2.1",
    url="https://github.com/erm0l0v/tta",

    author="Kirill Ermolov",
    author_email="erm0l0v@ya.ru",

    description="Time Tracker Autocompleter for Muranosoft",
    long_description=open('README.rst').read(),

    packages=['tta'],

    package_dir={'tta':
                 'tta'},
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,

    entry_points={
        'console_scripts': [
            'tta = tta.__main__:main',
        ]
    },

    keywords='Time Tracker, Autocomplete, Muranosoft',

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],

    test_suite='tests',
)



from __future__ import unicode_literals
from abc import ABCMeta, abstractmethod
from itertools import islice
import six
from git import Repo


def take(n, iterable):
    """Return first n items of the iterable as a list"""
    if isinstance(iterable, list) or isinstance(iterable, tuple):
        return iterable[:n]
    return list(islice(iterable, n))


@six.add_metaclass(ABCMeta)
class MonthWork(object):

    def __init__(self, email, patch=None):
        self.email = email
        self.patch = patch

    @abstractmethod
    def get_commits(self):
        pass

    def get_last_n_commits(self, n=25):
        commits = list(take(n, self.get_commits()))
        return reversed(commits)


class GitMonthWork(MonthWork):

    def filter_by_email(self, commit):
        return commit.author.email == self.email

    def get_message(self, commit):
        return commit.message.split('\n')[0]

    def get_commits(self):
        repo = Repo(self.patch)
        return map(self.get_message, filter(self.filter_by_email, repo.iter_commits()))


#!/usr/bin/env python

from __future__ import unicode_literals
import argparse
import operator
from .tt_integration import TTIntegration
from .month_work import GitMonthWork
from .worked_days import get_worked_days


categories = (
    (17, 'Client Support'),
    (12, 'Code Review'),
    (2, 'Development'),
    (4, 'Documenting'),
    (15, 'Graphic Design'),
    (27, 'Holiday'),
    (29, 'HR'),
    (14, 'Infrastructure'),
    (28, 'Marketing'),
    (7, 'Meeting'),
    (8, 'Miscellaneous'),
    (30, 'Office Management'),
    (35, 'Overtime leave'),
    (9, 'Project Management'),
    (10, 'Quality Assurance/Testing'),
    (3, 'Requirement Analysis'),
    (6, 'Research'),
    (25, 'Sick'),
    (31, 'Vacation non-paid'),
    (26, 'Vacation paid')
)


parser = argparse.ArgumentParser(description='Time Tracker Autocompleter for Muranosoft.')
parser.add_argument('-u', '--user',
                    help='Time Tracker username',
                    type=str,
                    required=True)
parser.add_argument('-p', '--password',
                    help='Time Tracker password',
                    type=str,
                    required=True)
parser.add_argument('-e', '--email',
                    help='Your email in git config',
                    type=str,
                    required=True)
parser.add_argument('-d', '--directory',
                    help='Path to git directory[optional]',
                    default=None)
parser.add_argument('-c', '--category',
                    help=('Time Tracker category, default: Development. Options:\n' +
                          '\n'.join(map(lambda i: '[ {0}\t{1} ]'.format(i[0], i[1]), categories))),
                    choices=list(map(operator.itemgetter(0), categories)),
                    type=int,
                    default=2)
parser.add_argument('--start_date',
                    help='Start date of period, Default: first day of current month',
                    type=str,
                    default=None)
parser.add_argument('--end_date',
                    help='End date of period, Default: last day of current month',
                    type=str,
                    default=None)
parser.add_argument('--start_work_day',
                    help='Hour of start working day. Default: 10',
                    type=int,
                    default=10)
parser.add_argument('--end_work_day',
                    help='Hour of end working day. Default: 18',
                    type=int,
                    default=18)


def main():
    options = parser.parse_args()
    tt = TTIntegration(options.user,
                       options.password,
                       options.category,
                       options.start_work_day,
                       options.end_work_day)
    month_work = GitMonthWork(options.email, options.directory)
    worked_days = get_worked_days(options.start_date, options.end_date)
    commits = month_work.get_last_n_commits(len(worked_days))
    data = zip(worked_days, commits)
    for (current_date, is_working), message in data:
        tt.post_message(current_date, is_working, message)

if __name__ == '__main__':
    main()